@@ -492,6 +492,7 @@ int relative_term_location(termNo, termsOnEdge, edgeLength, spacesPerPin)
  if space == 1, the progression is 1, 3, 5, 7 ...
  if space == 2, the progression is 1, 4, 7, 10 ...
 */
+
 /* ------------------------------------------------------------------------
  * add_in_out: add a signal to the list of terminals for a module
  * ------------------------------------------------------------------------
@@ -562,11 +563,57 @@ default: error("add_in_out: terminal with bad direction", name);
 	/* tie the term back into the list of terms on this net */
 
          tn->terms = (tlist *)concat_list(tt, tn->terms);
-
 }
 
-/* OLD:tt->y_pos = ((index % 2) == 1) ? tt->mod->y_size/2 - index/2	         */
-/*	                              : tt->mod->y_size/2 +  index/2 + 1;        */
+/* ------------------------------------------------------------------------
+ * add_xc_term: add a signal to the list of terminals for a module
+ * based on the XCircuit object for gate "name".
+ * ------------------------------------------------------------------------
+ */
+
+void add_xc_term(type, pinName, netName, index)
+    char *type;
+    char *pinName;
+    char *netName;
+    int index;
+{
+    tlist *tl;
+    term *tt;
+    net *tn;
+    objinstptr *xc_inst;
+    int result, x, y;
+    
+    /* Get the xcircuit object name corresponding to the module */
+    if (NameToObject(type, &xc_inst, FALSE) == NULL) return;
+
+    /* Find the location of the pin named "pinName" relative to the object position */
+    result = NameToPinLocation(xc_inst, pinName, &x, &y);
+    if (result < 0) {
+       add_in_out(netName, index, 1, 0);	/* backup behavior */
+       return;					/* Error---pin name not found */
+    }
+
+    tt = (term *)calloc(1, sizeof(term));
+
+    /* find this net in the global database */
+    tn = get_net(netName);
+    tt->nt = tn;    
+
+    tt->mod = curobj_module;
+    tt->name = (char *)calloc(1, TERM_NAME_SIZE);
+
+    sprintf(tt->name, "input_%d", index);
+    tt->side = LEFT;
+    tt->type = IN;
+    tt->x_pos = x;
+    tt->y_pos = y;
+    
+    curobj_module->terms = (tlist *)concat_list(tt, curobj_module->terms);
+
+    /* tie the term back into the list of terms on this net */
+
+    tn->terms = (tlist *)concat_list(tt, tn->terms);
+}
 
 /* ------------------------------------------------------------------------
  * add_special: add a special terminal to a module
