@@ -47,8 +47,6 @@ extern int *appcolors;
 /*----------------------------------------------*/
 #ifdef LGF
 
-Boolean lgfisloaded = False;
-
 /*----------------------------------------------*/
 /* loadlgf: Load an analog LGF file 		*/
 /*	mode = 1: import into current page;	*/
@@ -91,12 +89,11 @@ void loadlgf(int mode)
 
    /* Make sure that LGF object library has been loaded by loading it now. */
 
-   if (!lgfisloaded) {
+   if (check_library(LGF_LIB) < 0) {
       int ilib;
       strcpy(_STR, LGF_LIB);
       ilib = createlibrary();
       loadlibrary(ilib);
-      lgfisloaded = True;
    }
    
    /* Read header information */
@@ -248,9 +245,7 @@ void loadlgf(int mode)
 		  strptr->type = TEXT_STRING;
 		  strptr->data.string = (char *)malloc(1 + strlen(tstrp));
 		  strcpy(strptr->data.string, tstrp);
-#ifdef SCHEMA
 		  (*newlabel)->pin = NORMAL;
-#endif
 	       }
             }}
 	    break;
@@ -431,12 +426,10 @@ void loadlgf(int mode)
 			   labeldefaults(*newlabel, False, (*newinst)->position.x,
 				(*newinst)->position.y);
 			   (*newlabel)->color = DEFAULTCOLOR;
-#ifdef SCHEMA
 		  	   if (areastruct.schemon){
 			      (*newlabel)->pin = LOCAL;
 			      (*newlabel)->color = LOCALPINCOLOR;
 		  	   }
-#endif
 			   if (!strcmp(tmpstring, "TO"))
 			      (*newlabel)->position.x += ((flip) ? 48 : -48);
 			   else
