@@ -3,6 +3,17 @@
 /*     Master list of function prototypes				*/
 /*----------------------------------------------------------------------*/
 
+/* from undo.c */
+
+extern void register_for_undo(u_int, objinstptr, ...);	/* variable argument list */
+extern void undo_action();
+extern void redo_action();
+extern void flush_undo_stack();
+extern void flush_redo_stack();
+extern void free_undo_record(Undoptr);
+extern void free_redo_record(Undoptr);
+
+
 /* from tclxcircuit.c */
 
 #ifdef TCL_WRAPPER
@@ -128,10 +139,10 @@ extern void startrotate(xcWidget, pointertype, caddr_t);
 extern void startcopy(xcWidget, caddr_t, caddr_t);
 extern void startedit(xcWidget, caddr_t, caddr_t);
 extern void startdelete(xcWidget, caddr_t, caddr_t);
-extern void reviseselect(short *);
-extern void objectdelete(short);
+extern void reviseselect(short *, int, short *);
 extern void deletebutton(XButtonEvent *);
-extern short xc_undelete(xcWidget, pointertype, XButtonEvent *);
+extern short *xc_undelete(objinstptr, objectptr, short);
+extern objectptr delete_element(objinstptr, short *, int, short);
 extern void printname(objectptr);
 extern Boolean checkname(objectptr);
 extern objectptr finddot();
@@ -151,6 +162,9 @@ extern void releasebutton(XButtonEvent *);
 extern void pathbutton(genericptr *, XButtonEvent *);
 extern void resizearea(xcWidget, caddr_t, caddr_t);
 extern void drawarea(xcWidget, caddr_t, caddr_t);
+extern void standard_element_delete(short);
+extern void delete_for_xfer(short);
+extern void delete_noundo(short);
 
 /* from filelist.c: */
 
@@ -171,6 +185,10 @@ extern int lookdirectory(char *);
 
 /* from files.c: */
 
+#ifdef ASG
+extern void importspice();
+#endif
+
 extern char *ridnewline(char *);
 extern void free_single(genericptr *);
 extern void reset(objectptr, short);
@@ -421,6 +439,7 @@ extern void promptparam(xcWidget, caddr_t, caddr_t);
 #endif
 extern void stringparam(xcWidget, caddr_t, caddr_t);
 extern void toggle(xcWidget, Boolean *, caddr_t);
+extern int setelementstyle(xcWidget, u_short, u_short);
 extern void setcolorscheme(Boolean);
 extern void inversecolor(xcWidget, Boolean *, caddr_t);
 extern void togglegrid(u_short);
@@ -437,11 +456,13 @@ extern void renamelib(short);
 extern void setcolormark(int);
 extern void setallstylemarks(u_short);
 extern polyptr checkforbbox(objectptr);
+#ifndef TCL_WRAPPER
 extern void setfill(xcWidget, pointertype, caddr_t);
 extern void makebbox(xcWidget, pointertype, caddr_t);
 extern void setclosure(xcWidget, pointertype, caddr_t);
-extern void setline(xcWidget, pointertype, caddr_t);
 extern void setopaque(xcWidget, pointertype, caddr_t);
+#endif
+extern int setline(xcWidget, pointertype, caddr_t);
 #ifdef TCL_WRAPPER
 extern void setcolor(xcWidget, int);
 #else
