@@ -84,21 +84,17 @@ Boolean checkselect(short value)
    else return True;
 }
 
-/*-----------------------------------------------------------*/
-/* Select list numbering revision for routine objectdelete() */
-/*-----------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Select list numbering revision when an element is deleted	*/
+/* from an object. 						*/
+/*--------------------------------------------------------------*/
 
-void reviseselect(short *selectobj)
+void reviseselect(short *slist, int selects, short *removed)
 {
    short *chkselect;
 
-   for (chkselect = selectobj + 1; chkselect < areastruct.selectlist
-	  + areastruct.selects; chkselect++)
-      if (*chkselect > *selectobj) (*chkselect)--;
-
-   for (chkselect = areastruct.selectlast; chkselect < areastruct.selectlast
-	  + areastruct.lastselects; chkselect++)
-      if (*chkselect > *selectobj) (*chkselect)--;
+   for (chkselect = slist; chkselect < slist + selects; chkselect++)
+      if (*chkselect > *removed) (*chkselect)--;
 }
 
 /*----------------------*/
@@ -826,13 +822,9 @@ void drawselects(objectptr thisobj, objinstptr thisinst)
 
 void freeselects()
 {
-   if (areastruct.selects > 0) {
-      if (areastruct.lastselects > 0)
-         free(areastruct.selectlast);
-      areastruct.selectlast = areastruct.selectlist;
-      areastruct.lastselects = areastruct.selects;
-      areastruct.selects = 0;
-   }
+   if (areastruct.selects > 0)
+      free(areastruct.selectlist);
+   areastruct.selects = 0;
 }
 
 /*--------------------------------------------------------------*/
@@ -850,23 +842,6 @@ void clearselects()
    }
 }
 
-/*--------------------------------------------------------------*/
-/* Free memory from both the current and previous selection	*/
-/* lists, because a page change has invalidated both.		*/
-/*--------------------------------------------------------------*/
-
-void destroyselects()
-{
-   if (areastruct.selects > 0) free(areastruct.selectlist);
-   if (areastruct.lastselects > 0) free(areastruct.selectlast);
-   areastruct.selects = 0;
-   areastruct.lastselects = 0;
- 
-   setallstylemarks(areastruct.style);
-   setcolormark(areastruct.color);
-   setdefaultfontmarks();
-}
-
 /*--------------------------------------------------------------*/
 /* Unselect all the selected objects and free memory from the	*/
 /* selection list.						*/
