@@ -2998,7 +2998,7 @@ int main(int argc, char **argv)
    /*---------------------------*/
 
    for (k = argc - 1; k > 0; k--) {
-      if (!strncmp(argv[k], SCHEM_FLAG, strlen(SCHEM_FLAG))) {
+      if (!strncmp(argv[k], "-schem", 6)) {
 	 areastruct.schemon = True;
 	 break;
       }
