@@ -27,6 +27,13 @@ typedef unsigned long	u_long;
   #define Flush fflush
 #endif
 
+#ifdef TCL_WRAPPER
+  #define malloc Tcl_Alloc
+  #define calloc(a,b) Tcl_Alloc(a * b)
+  #define free(a) Tcl_Free((char *)(a))
+  #define realloc(a,b) Tcl_Realloc((char *)(a), b)
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Definition shortcuts 						*/
 /*----------------------------------------------------------------------*/
@@ -36,6 +43,11 @@ typedef unsigned long	u_long;
 #define sign(a)			((a) <= 0 ? -1 : 1)
 #define max(a,b)		((a) < (b) ? (b) : (a))
 #define min(a,b)		((a) < (b) ? (a) : (b))
+
+/*----------------------------------------------------------------------*/
+/* Deal with 32/64 bit processors based on autoconf results.		*/
+/*----------------------------------------------------------------------*/
+
 #if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
 #define Number(a)		(void *)((u_int) a)
 typedef u_int pointertype;
@@ -151,10 +163,7 @@ typedef struct {
 #define MINAUTOSCALE 0.75 /* Won't automatically scale closer than this */
 #define MAXCHANGES 20 /* Number of changes to induce a temp file save	*/
 #define STIPPLES    8 /* Number of predefined stipple patterns		*/
-
-#ifdef SCHEMA
 #define PADSPACE   10 /* Spacing of pinlabels from their origins	*/
-#endif
 
 #ifdef HAVE_XPM
 #define TBBORDER   1  /* border around toolbar buttons */
@@ -198,13 +207,7 @@ CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
 TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, POLY_MODE, BOX0_MODE,
 BOX_MODE, EBOX_MODE, ARC0_MODE, ARC_MODE, EARC_MODE, SPLINE0_MODE,
 SPLINE_MODE, ESPLINE_MODE, EPATH_MODE, LPARAM_MODE, IPARAM_MODE,
-ULPARAM_MODE, UIPARAM_MODE
-
-#ifdef SCHEMA
-, CONNECT_MODE, ASSOC_MODE
-#endif
-
-};
+ULPARAM_MODE, UIPARAM_MODE, CONNECT_MODE, ASSOC_MODE};
 
 /*----------------------------------------------------------------------*/
 /* File loading modes							*/
@@ -212,7 +215,7 @@ ULPARAM_MODE, UIPARAM_MODE
 
 enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #ifdef ASG
-	IMPORTIVF, IMPORTSPICE,
+	IMPORTSPICE,
 #endif
 	LOAD_MODES
 };
@@ -225,9 +228,14 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define RIGHT		2
 #define NOTBOTTOM	4
 #define TOP		8
-#define FLIPINV		16	/* 1 if text is flip-invariant */
-#define PINVISIBLE	32	/* 1 if pin visible outside of object */
-#define PINNOOFFSET	64	/* 0 if pin label offset from position */
+#define FLIPINV		16	/* 1 if text is flip-invariant		*/
+#define PINVISIBLE	32	/* 1 if pin visible outside of object	*/
+#define PINNOOFFSET	64	/* 0 if pin label offset from position	*/
+#define LATEXLABEL	128
+
+#define RLJUSTFIELD	3	/* right-left justification bit field	*/
+#define TBJUSTFIELD	12	/* top-bottom justification bit field	*/
+#define NONJUSTFIELD	240	/* everything but justification fields	*/
 
 /*----------------------------------------------------------------------*/
 /* Text string part: types						*/
@@ -331,12 +339,11 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 /* Schematic object types and pin label types	   			*/
 /*----------------------------------------------------------------------*/
 
-#ifdef SCHEMA
-
-#define SCHEMATIC 0		/* Schematic on page */
-#define TRIVIAL 1		/* Symbol as non-schematic element */
-#define SYMBOL 2		/* Symbol associated with a schematic */
-#define FUNDAMENTAL 3		/* Standalone symbol */
+#define PRIMARY 0		/* Primary (master) schematic page */
+#define SECONDARY 1		/* Secondary (slave) schematic page */
+#define TRIVIAL 2		/* Symbol as non-schematic element */
+#define SYMBOL 3		/* Symbol associated with a schematic */
+#define FUNDAMENTAL 4		/* Standalone symbol */
 
 #define LOCAL 1
 #define GLOBAL 2
@@ -344,8 +351,6 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 
 #define HIERARCHY_LIMIT 256	/* Stop if recursion goes this deep */
 
-#endif
-
 /*----------------------------------------------------------------------*/
 /* Save types.	This list incorporates "ALL_PAGES", below.		*/
 /*----------------------------------------------------------------------*/
@@ -405,11 +410,13 @@ typedef XfPoint* fpointlist;
 #define PATH	32
 #define SEL_ANY	63  /* for selection mechanism only */
 
+#define REMOVE_TAG 64  /* add this value to objects to be removed */
+
 /*----------------------------------------------------------------------*/
 /* Allowed parameterization types					*/
 /*----------------------------------------------------------------------*/
 
-#define P_POSITION	0	/* mode only, not a real parameter */
+#define P_NUMERIC	0	/* uncommitted numeric parameter */
 #define P_SUBSTRING	1
 #define P_POSITION_X	2
 #define P_POSITION_Y	3
@@ -423,6 +430,8 @@ typedef XfPoint* fpointlist;
 #define P_SCALE		11
 #define P_LINEWIDTH	12
 #define P_COLOR		13
+#define P_EXPRESSION	14
+#define P_POSITION	15	/* mode only, not a real parameter */
 
 /*----------------------------------------------------------------------*/
 /* Labels are constructed of strings and executables 			*/
@@ -437,7 +446,6 @@ typedef struct _stringpart {
      u_char	*string;
      int	color;
      int	font;
-     int	paramno;
      float	scale;
      short	kern[2];
    }		data;
@@ -452,22 +460,41 @@ typedef struct _stringpart {
 /* (Add types as necessary)						*/
 /*----------------------------------------------------------------------*/
 
-enum paramtypes {XC_INT = 0, XC_FLOAT, XC_STRING};
+enum paramtypes {XC_INT = 0, XC_FLOAT, XC_STRING, XC_EXPR};
 
-typedef struct {
+/* Object parameters: general key:value parameter model */
+/* Note that this really should be a hash table, not a linked list. . . */
+
+typedef struct _oparam *oparamptr;
+
+typedef struct _oparam {
+   char *	key;		/* name of the parameter */
    u_char	type;		/* type is from paramtypes list above */
    u_char	which;		/* what the parameter represents (P_*) */
    union {
-      stringpart *string;
+      stringpart *string;	/* xcircuit label type */
+      char	 *expr;		/* TCL (string) expression */
       int	  ivalue;	/* also covers type short int by typecasting */
       float	  fvalue;
    } parameter;			/* default or substitution value */
-} oparam, *oparamptr;
+   oparamptr	next;		/* next parameter in linked list */
+} oparam;
 
-typedef struct {
-   u_char	paramno;	/* parameter number to get value from.	   */
-   u_char	pointno;	/* point number in point array, for polygons */
-} eparam, *eparamptr;
+/* Element parameters: reference back to the object's parameters */
+/* These parameters are forward-substituted when descending into */
+/* an object instance.						 */
+/* Note that this really should be a hash table, not a linked list. . . */
+
+typedef struct _eparam *eparamptr;
+
+typedef struct _eparam {
+   char *	key;	    /* name of the parameter */
+   union {
+      int	pointno;    /* point number in point array, for polygons */
+      char	*refkey;    /* parameter reference key, for instances */
+   } pdata;
+   eparamptr	next;	    /* next parameter in linked list */
+} eparam;
 
 /*----------------------------------------------------------------------*/
 /* Generic element type	is a superset of all elements.			*/	
@@ -477,7 +504,6 @@ typedef struct {
    u_char	type;		/* type is LABEL, POLYGON, etc., from below */
    int		color;
    eparamptr	passed;
-   u_char	num_params;
 } generic, *genericptr;		/* (convenience function for retypecasting) */
 
 /*----------------------------------------------------------------------*/
@@ -499,16 +525,13 @@ typedef struct {
    u_char	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    XPoint	position;
    short	rotation;
    float	scale;
    objectptr	thisobject;
-   oparamptr    *params;	/* parameter substitutions for this instance */
+   oparamptr	params;		/* parameter substitutions for this instance */
    BBox		bbox;		/* per-instance bounding box information */
-#ifdef SCHEMA
    BBox		*schembbox;	/* Extra bounding box for pin labels */
-#endif
 } objinst, *objinstptr;
 
 /* Linked-list for objects */
@@ -549,14 +572,11 @@ typedef struct {
    u_char	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    XPoint	position;
    short	rotation;
    float	scale;
    short	justify;
-#ifdef SCHEMA
    u_char	pin;
-#endif
    stringpart	*string;
 } label, *labelptr;
 
@@ -568,7 +588,6 @@ typedef struct {
    u_char	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    short	number;
@@ -583,7 +602,6 @@ typedef struct {
    u_char	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    XPoint	ctrl[4];
@@ -599,7 +617,6 @@ typedef struct {
    u_char	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    short	radius; 	/* x-axis radius */
@@ -620,7 +637,6 @@ typedef struct {
    u_char	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    short	parts;
@@ -631,15 +647,14 @@ typedef struct {
 /* Netlist structures for schematic capture				*/
 /*----------------------------------------------------------------------*/
 
-#ifdef SCHEMA
-
 /* Linked polygon list */
 
 typedef struct _Polylist *PolylistPtr;
 typedef struct _Polylist
 {
+   objectptr cschem;	/* Schematic containing the polygon */
    polyptr poly;
-   PolylistPtr next;
+   PolylistPtr next;	/* Next polygon in the linked list */
 } Polylist;
 
 /* Linked label list */
@@ -647,6 +662,7 @@ typedef struct _Polylist
 typedef struct _Labellist *LabellistPtr;
 typedef struct _Labellist
 {
+   objectptr cschem;	/* Schematic containing the label */
    labelptr label;
    LabellistPtr next;
 } Labellist;
@@ -679,8 +695,9 @@ typedef struct _Portlist
 typedef struct _Calllist *CalllistPtr;
 typedef struct _Calllist
 {
-   objectptr   callobj;
-   objinstptr  callinst;
+   objectptr   cschem;		/* Schematic page containing the instance */
+   objinstptr  callinst;	/* Instance called */
+   objectptr   callobj;		/* Object of instance called */
    int         devindex;	/* if non-zero, index of device in netlist */
    PortlistPtr ports;
    CalllistPtr next;
@@ -700,6 +717,7 @@ struct Pstr {
 };
 
 struct Ptab {
+   objectptr cschem;
    struct Pnet *nets;
    struct Pstr *pins;
    struct Ptab *next;
@@ -714,8 +732,6 @@ typedef struct {
    objinstptr	thisinst;
 } Highlight;
 
-#endif
-
 /*----------------------------------------------------------------------*/
 /* Main object structure						*/
 /*----------------------------------------------------------------------*/
@@ -730,21 +746,17 @@ typedef struct _xcobject {
 				/* parameterized elements) */
    short	parts;
    genericptr	*plist;	 	/* to be retypecast to label, polygon, etc. */
-   u_char	num_params;	/* number of calling parameters */
-   oparamptr	*params;	/* list of parameter defaults */
+   oparamptr	params;		/* list of parameters, with default values */
    
-#ifdef SCHEMA
    Highlight	highlight;	/* net to be highlighted on redraw */
    u_char	schemtype;
    objectptr	symschem;	/* schematic page support */
-
    char		*devname;	/* if non-null, name of device in netlist */
    Boolean	valid;		/* Is current netlist valid? */
    Boolean	traversed;	/* Flag to indicate object was processed */
    NetlistPtr   netlist;
    PortlistPtr  portlist;
    CalllistPtr  calllist;
-#endif
 } object;
 
 /*----------------------------------------------------------------------*/
@@ -877,10 +889,8 @@ enum {DIRECTORY = 0, MATCH, NONMATCH};	/* file types */
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-#ifdef SCHEMA
    /* schematic layout colors */
-   Pixel	globalcolor, localcolor, infocolor;
-#endif
+   Pixel	globalcolor, localcolor, infocolor, ratsnestcolor;
 
    /* non-schematic layout color(s) */
    Pixel	bboxpix;
@@ -899,7 +909,12 @@ typedef struct {
    Pixel	auxpix2, barpix2, parampix2;
 
    int		width, height, timeout;
-   XFontStruct	*xcfont, *helpfont, *filefont, *textfont, *titlefont;
+   XFontStruct	*helpfont, *filefont;
+
+#ifndef TCL_WRAPPER
+   XFontStruct	*xcfont, *textfont, *titlefont;
+#endif
+
 } ApplicationData, *ApplicationDataPtr;
 
 /*----------------------------------------------------------------------*/
@@ -981,29 +996,27 @@ enum {
    XCF_ENDDATA = -2, XCF_SPACER,
    XCF_Page, XCF_Justify, XCF_Superscript, XCF_Subscript, XCF_Normalscript,
    XCF_Nextfont, XCF_Boldfont, XCF_Italicfont, XCF_Normalfont, XCF_Underline,
-   XCF_Overline, XCF_ISO_Encoding, XCF_Return, XCF_Halfspace, XCF_Quarterspace,
+   XCF_Overline, XCF_ISO_Encoding, XCF_Halfspace, XCF_Quarterspace,
    XCF_Special, XCF_TabStop, XCF_TabForward, XCF_TabBackward,
    XCF_Text_Return, XCF_Text_Delete, XCF_Text_Right, XCF_Text_Left,
    XCF_Text_Up, XCF_Text_Down, XCF_Text_Split,
-   XCF_Text_Home, XCF_Text_End, XCF_Return_Char, XCF_Parameter,
-   XCF_Edit_Param, XCF_Edit_Break, XCF_Edit_Delete, XCF_Edit_Insert,
-   XCF_Edit_Next, XCF_Attach, XCF_Next_Library, XCF_Library_Directory,
+   XCF_Text_Home, XCF_Text_End, XCF_Linebreak, XCF_Parameter,
+   XCF_Edit_Param,XCF_Edit_Break, XCF_Edit_Delete, XCF_Edit_Insert,
+   XCF_Edit_Append, XCF_Edit_Next, XCF_Attach,
+   XCF_Next_Library, XCF_Library_Directory,
    XCF_Library_Move, XCF_Library_Copy, XCF_Library_Edit, XCF_Library_Delete,
    XCF_Library_Duplicate, XCF_Library_Hide, XCF_Library_Virtual,
-   XCF_Page_Directory, XCF_Library_Pop,
+   XCF_Page_Directory, XCF_Library_Pop, XCF_Virtual,
    XCF_Help, XCF_Redraw, XCF_View, XCF_Zoom_In, XCF_Zoom_Out, XCF_Pan,
    XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
    XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
    XCF_SnapTo, XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc,
    XCF_Text, XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline,
-   XCF_Edit, XCF_Undelete, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
+   XCF_Edit, XCF_Undo, XCF_Redo, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
    XCF_Dotted, XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start,
    XCF_Finish, XCF_Cancel, XCF_Nothing, XCF_Exit,
-
-#ifdef SCHEMA
    XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Pin_Global, XCF_Info_Label,
    XCF_Connectivity, XCF_Sim, XCF_SPICE, XCF_PCB, XCF_SPICEflat,
-#endif
    NUM_FUNCTIONS
 };
 
@@ -1068,10 +1081,8 @@ typedef struct {
    Boolean	mapped;		/* indicates if window is drawable */
    Boolean	filefilter;	/* Is the file list filtered? */
    Boolean	editinplace;
-#ifdef SCHEMA
    Boolean	schemon;
    Boolean	pinpointon;
-#endif
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
 #endif
@@ -1080,9 +1091,7 @@ typedef struct {
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
-   short	lastselects;
    short	lastlibrary;
-   short	*selectlast;
    objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
@@ -1092,6 +1101,20 @@ typedef struct {
    char		*lastbackground;
 } Clientdata;
 
+/* Record for undo function */ 
+
+typedef struct _undostack *Undoptr;
+
+typedef struct _undostack {
+   Undoptr next;	/* next record in undo stack */
+   Undoptr last;	/* double-linked for "redo" function */
+   u_int type;		/* type of event */
+   objinstptr thisinst;	/* instance of object in which event occurred */
+   int idata;		/* simple undedicated integer datum */
+   char *undodata;	/* free space to be malloc'd */
+			/* (size dependent on "type") */
+} Undostack;
+
 /* This whole thing needs to be cleaned up. . . now I've got objects in */
 /* "library" which are repeated both in the "instlist" pair and the	*/
 /* "thisobject" pointer in each instance.  The instance is repeated on	*/
@@ -1117,7 +1140,8 @@ typedef struct {
    short	pages;
    short	numlibs;
    Pagedata	**pagelist;
-   Library	delbuffer;
+   Undoptr	undostack;	/* actions to undo */
+   Undoptr	redostack;	/* actions to redo */
    Library	fontlib;
    Library	*userlibs;
    objinstptr   *libtop;
